@@ -16,11 +16,11 @@ pub use web_sys::WebGlUniformLocation as UniformLocation;
 pub use web_sys::WebGlVertexArrayObject as VertexArrayObject;
 
 #[derive(Clone)]
-pub struct Context {
+pub struct GLContext {
     inner: std::rc::Rc<InnerGl>,
 }
 
-impl Context {
+impl GLContext {
     pub fn new(webgl_context: InnerGl) -> Self {
         Self {
             inner: std::rc::Rc::new(webgl_context),
@@ -101,6 +101,17 @@ impl Context {
     pub fn create_shader(&self, type_: ShaderType) -> Option<Shader> {
         self.inner.create_shader(type_.to_const())
     }
+    pub fn get_shader_info_log(&self, shader: &Shader) -> Option<String> {
+        if let Some(log) = self.inner.get_shader_info_log(shader) {
+            if log.len() > 0 {
+                Some(log)
+            } else {
+                None
+            }
+        } else {
+            None
+        }
+    }
 
     pub fn compile_shader(&self, source: &str, shader: &Shader) {
         let header = "#version 300 es\nprecision highp float;\nprecision highp int;\nprecision highp sampler2DArray;\n";
@@ -711,7 +722,7 @@ impl Context {
     }
 }
 
-impl std::ops::Deref for Context {
+impl std::ops::Deref for GLContext {
     type Target = InnerGl;
 
     fn deref(&self) -> &InnerGl {
