@@ -1,4 +1,4 @@
-use crate::context::{consts, AttributeLocation, Context, ShaderType};
+use crate::context::{consts, AttributeLocation, ShaderType};
 use crate::core::*;
 use std::cell::RefCell;
 use std::collections::HashMap;
@@ -26,7 +26,7 @@ impl Program {
         context: &Context,
         vertex_shader_source: &str,
         fragment_shader_source: &str,
-    ) -> Result<Program> {
+    ) -> ThreeDResult<Program> {
         let vert_shader = context
             .create_shader(ShaderType::Vertex)
             .ok_or(CoreError::ShaderCreation)?;
@@ -42,15 +42,15 @@ impl Program {
         let success = context.link_program(&id);
 
         if !success {
-            if let Some(log) = context.get_program_info_log(&id) {
-                Err(CoreError::ShaderLink(log))?;
-            }
             if let Some(log) = context.get_shader_info_log(&vert_shader) {
                 Err(CoreError::ShaderCompilation("vertex".to_string(), log))?;
             }
             if let Some(log) = context.get_shader_info_log(&frag_shader) {
                 Err(CoreError::ShaderCompilation("fragment".to_string(), log))?;
             }
+            if let Some(log) = context.get_program_info_log(&id) {
+                Err(CoreError::ShaderLink(log))?;
+            }
             unreachable!();
         }
 
@@ -95,7 +95,7 @@ impl Program {
     /// Send the given integer value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform int`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_int(&self, name: &str, data: &i32) -> Result<()> {
+    pub fn use_uniform_int(&self, name: &str, data: &i32) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
         self.context.uniform1i(location, *data);
         self.context.unuse_program();
@@ -106,7 +106,7 @@ impl Program {
     /// Send the given float value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform float`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_float(&self, name: &str, data: &f32) -> Result<()> {
+    pub fn use_uniform_float(&self, name: &str, data: &f32) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
         self.context.uniform1f(location, *data);
         self.context.unuse_program();
@@ -117,9 +117,9 @@ impl Program {
     /// Send the given [Vec2](crate::Vec2) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec2`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec2(&self, name: &str, data: &Vec2) -> Result<()> {
+    pub fn use_uniform_vec2(&self, name: &str, data: &Vec2) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
-        self.context.uniform2fv(location, &mut [data.x, data.y]);
+        self.context.uniform2fv(location, &mut data.to_slice());
         self.context.unuse_program();
         Ok(())
     }
@@ -128,10 +128,9 @@ impl Program {
     /// Send the given [Vec3](crate::Vec3) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec3`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec3(&self, name: &str, data: &Vec3) -> Result<()> {
+    pub fn use_uniform_vec3(&self, name: &str, data: &Vec3) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
-        self.context
-            .uniform3fv(location, &mut [data.x, data.y, data.z]);
+        self.context.uniform3fv(location, &mut data.to_slice());
         self.context.unuse_program();
         Ok(())
     }
@@ -140,10 +139,20 @@ impl Program {
     /// Send the given [Vec4](crate::Vec4) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec4`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec4(&self, name: &str, data: &Vec4) -> Result<()> {
+    pub fn use_uniform_vec4(&self, name: &str, data: &Vec4) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
-        self.context
-            .uniform4fv(location, &mut [data.x, data.y, data.z, data.w]);
+        self.context.uniform4fv(location, &mut data.to_slice());
+        self.context.unuse_program();
+        Ok(())
+    }
+
+    ///
+    /// Send the given [Quat](crate::Quat) value to this shader program and associate it with the given named variable.
+    /// The glsl shader variable must be of type `uniform vec4`, meaning it is uniformly available across all processing of vertices and fragments.
+    ///
+    pub fn use_uniform_quat(&self, name: &str, data: &Quat) -> ThreeDResult<()> {
+        let location = self.get_uniform_location(name)?;
+        self.context.uniform4fv(location, &mut data.to_slice());
         self.context.unuse_program();
         Ok(())
     }
@@ -152,7 +161,7 @@ impl Program {
     /// Send the given [Mat2](crate::Mat2) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat2`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat2(&self, name: &str, data: &Mat2) -> Result<()> {
+    pub fn use_uniform_mat2(&self, name: &str, data: &Mat2) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix2fv(location, &mut data.to_slice());
@@ -164,7 +173,7 @@ impl Program {
     /// Send the given [Mat3](crate::Mat3) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat3`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat3(&self, name: &str, data: &Mat3) -> Result<()> {
+    pub fn use_uniform_mat3(&self, name: &str, data: &Mat3) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix3fv(location, &mut data.to_slice());
@@ -176,7 +185,7 @@ impl Program {
     /// Send the given [Mat4](crate::Mat4) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat4`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat4(&self, name: &str, data: &Mat4) -> Result<()> {
+    pub fn use_uniform_mat4(&self, name: &str, data: &Mat4) -> ThreeDResult<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix4fv(location, &mut data.to_slice());
@@ -184,7 +193,7 @@ impl Program {
         Ok(())
     }
 
-    fn get_uniform_location(&self, name: &str) -> Result<&crate::context::UniformLocation> {
+    fn get_uniform_location(&self, name: &str) -> ThreeDResult<&crate::context::UniformLocation> {
         self.set_used();
         let loc = self
             .uniforms
@@ -197,7 +206,7 @@ impl Program {
     /// Use the given [Texture2D] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform sampler2D` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture(&self, name: &str, texture: &impl Texture) -> Result<()> {
+    pub fn use_texture(&self, name: &str, texture: &impl Texture) -> ThreeDResult<()> {
         let index = self.get_texture_index(name);
         texture.bind(index);
         self.use_uniform_int(name, &(index as i32))?;
@@ -208,7 +217,7 @@ impl Program {
     /// Use the given [TextureArray] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform sampler2DArray` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture_array(&self, name: &str, texture: &impl TextureArray) -> Result<()> {
+    pub fn use_texture_array(&self, name: &str, texture: &impl TextureArray) -> ThreeDResult<()> {
         let index = self.get_texture_index(name);
         texture.bind(index);
         self.use_uniform_int(name, &(index as i32))?;
@@ -219,7 +228,7 @@ impl Program {
     /// Use the given [TextureCube] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform samplerCube` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture_cube(&self, name: &str, texture: &impl TextureCube) -> Result<()> {
+    pub fn use_texture_cube(&self, name: &str, texture: &impl TextureCube) -> ThreeDResult<()> {
         let index = self.get_texture_index(name);
         texture.bind(index);
         self.use_uniform_int(name, &(index as i32))?;
@@ -257,7 +266,7 @@ impl Program {
     /// Each value in the buffer is used when rendering one vertex using the [Program::draw_arrays] or [Program::draw_elements] methods.
     /// Therefore the buffer must contain the same number of values as the number of vertices specified in those draw calls.
     ///
-    pub fn use_attribute(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
+    pub fn use_attribute(&self, name: &str, buffer: &VertexBuffer) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -276,7 +285,7 @@ impl Program {
     /// Each value in the buffer is used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain the same number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+    pub fn use_attribute_instanced(&self, name: &str, buffer: &InstanceBuffer) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -295,7 +304,7 @@ impl Program {
     /// Each contiguous 2 values in the buffer are used when rendering one vertex using the [Program::draw_arrays] or [Program::draw_elements] methods.
     /// Therefore the buffer must contain 2 times the number of values as the number of vertices specified in those draw calls.
     ///
-    pub fn use_attribute_vec2(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
+    pub fn use_attribute_vec2(&self, name: &str, buffer: &VertexBuffer) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -314,7 +323,11 @@ impl Program {
     /// Each contiguous 2 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain 2 times the number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_vec2_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+    pub fn use_attribute_vec2_instanced(
+        &self,
+        name: &str,
+        buffer: &InstanceBuffer,
+    ) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -333,7 +346,7 @@ impl Program {
     /// Each contiguous 3 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain 3 times the number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_vec3(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
+    pub fn use_attribute_vec3(&self, name: &str, buffer: &VertexBuffer) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(&name)?;
@@ -352,7 +365,11 @@ impl Program {
     /// Each contiguous 3 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain 3 times the number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_vec3_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+    pub fn use_attribute_vec3_instanced(
+        &self,
+        name: &str,
+        buffer: &InstanceBuffer,
+    ) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(&name)?;
@@ -371,7 +388,7 @@ impl Program {
     /// Each contiguous 4 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain 4 times the number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_vec4(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
+    pub fn use_attribute_vec4(&self, name: &str, buffer: &VertexBuffer) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -390,7 +407,11 @@ impl Program {
     /// Each contiguous 4 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
     /// Therefore the buffer must contain 4 times the number of values as the number of instances specified in those draw calls.
     ///
-    pub fn use_attribute_vec4_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+    pub fn use_attribute_vec4_instanced(
+        &self,
+        name: &str,
+        buffer: &InstanceBuffer,
+    ) -> ThreeDResult<()> {
         if buffer.count() > 0 {
             buffer.bind();
             let loc = self.location(name)?;
@@ -520,7 +541,21 @@ impl Program {
         self.context.unuse_program();
     }
 
-    fn location(&self, name: &str) -> Result<AttributeLocation> {
+    ///
+    /// Returns true if this program uses the uniform with the given name.
+    ///
+    pub fn requires_uniform(&self, name: &str) -> bool {
+        self.uniforms.contains_key(name)
+    }
+
+    ///
+    /// Returns true if this program uses the attribute with the given name.
+    ///
+    pub fn requires_attribute(&self, name: &str) -> bool {
+        self.vertex_attributes.contains_key(name)
+    }
+
+    fn location(&self, name: &str) -> ThreeDResult<AttributeLocation> {
         self.set_used();
         let location = self
             .vertex_attributes
