@@ -23,7 +23,11 @@ impl FlyControl {
         }
     }
 
-    pub fn handle_events(&mut self, camera: &mut Camera, events: &mut [Event]) -> Result<bool> {
+    pub fn handle_events(
+        &mut self,
+        camera: &mut Camera,
+        events: &mut [Event],
+    ) -> ThreeDResult<bool> {
         self.control.handle_events(camera, events)
     }
 }
